from django.db import models
from datetime import datetime, timedelta, date


def today():
    return date.today()


class Product(models.Model):
    product = models.CharField(max_length=128)
    lead_time_days = models.IntegerField()
    colour = models.CharField(max_length=64, blank=True)
    size = models.CharField(max_length=64, blank=True)
    cnc = models.IntegerField()
    sanding = models.IntegerField()
    painting = models.IntegerField()
    assembly = models.IntegerField()
    quality_control = models.IntegerField()
    packaging = models.IntegerField()
    mirror = models.IntegerField()
    part_total = models.IntegerField()
    in_stock = models.IntegerField(default=0)

    def __str__(self):
        return self.product


class List(models.Model):
    item = models.CharField(max_length=512)
    completed = models.BooleanField(default=False)

    def __str__(self):
        return self.item


#################################################################################
# Redundant code that was for single audio alert function. Can delete id django allows
class ToDo(models.Model):
    item = models.CharField(max_length=512)
    order_num = models.CharField(max_length=64)
    customer = models.CharField(max_length=64)
    product = models.CharField(max_length=128)
    quantity = models.IntegerField()
    from_stock = models.IntegerField()
    to_make = models.IntegerField()
    created_on = models.DateTimeField(auto_now_add=True)
    completed = models.BooleanField(default=False)

    def __str__(self):
        return self.item


#################################################################################

class TaskList(models.Model):
    item = models.CharField(max_length=512)
    order_num = models.CharField(max_length=64)
    customer = models.CharField(max_length=64)
    product = models.CharField(max_length=128)
    product_order = models.CharField(max_length=8, default=0)
    quantity = models.IntegerField()
    quantity_order = models.CharField(max_length=8, default=0)
    from_stock = models.IntegerField()
    from_stock_order = models.CharField(max_length=8, default=0)
    to_make = models.IntegerField()
    to_make_order = models.CharField(max_length=8, default=0)
    original_pk = models.CharField(max_length=8, default=0)
    max_lead_time = models.IntegerField(default=1)
    part_total = models.IntegerField(default=1)
    created_on = models.DateTimeField(auto_now_add=True)
    modified_on = models.DateTimeField(auto_now=True)
    start_on = models.DateField()
    due_date = models.DateField()
    overdue_date = models.DateField()
    day_1 = models.CharField(max_length=16, default=0)
    day_2 = models.CharField(max_length=16, default=0)
    day_3 = models.CharField(max_length=16, default=0)
    day_4 = models.CharField(max_length=16, default=0)
    day_5 = models.CharField(max_length=16, default=0)
    task_item_completed = models.BooleanField(default=False)
    audio_alert_active = models.BooleanField(default=True)
    completed = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        from_lead = self.max_lead_time
        now_due = datetime.strptime(str(self.start_on), "%Y-%m-%d") + timedelta(days=from_lead)
        now_overdue = datetime.strptime(str(self.start_on), "%Y-%m-%d") + timedelta(days=from_lead + 1)

        date_list = []
        from_start_after = str(self.start_on).split('-')
        current = datetime(int(from_start_after[0]), int(from_start_after[1]), int(from_start_after[2]))
        end = datetime(int(from_start_after[0]), int(from_start_after[1]), int(from_start_after[2]))
        end += timedelta(days=45)
        while current < end:
            if current.weekday() < 5:
                date_list.append(current.strftime("%Y-%m-%d"))
            current += timedelta(days=1)
        for value in BuildOrder.objects.all():
            if value.max_lead_time == 1:
                now_due = date_list[1]
                now_overdue = date_list[2]
            if value.max_lead_time == 2:
                now_due = date_list[2]
                now_overdue = date_list[3]
            if value.max_lead_time == 3:
                now_due = date_list[3]
                now_overdue = date_list[4]
            if value.max_lead_time == 4:
                now_due = date_list[4]
                now_overdue = date_list[5]
            if value.max_lead_time == 5:
                now_due = date_list[5]
                now_overdue = date_list[6]

        if self.due_date is not None:
            now_due = self.due_date
        if self.overdue_date is not None:
            now_overdue = self.overdue_date

        self.due_date = now_due
        self.overdue_date = now_overdue
        super(TaskList, self).save(*args, **kwargs)

    def __str__(self):
        return self.item


class Company(models.Model):
    company = models.CharField(max_length=64)
    unique_id = models.CharField(max_length=64)
    customer = models.CharField(max_length=128, default='VALUE WILL AUTO POPULATE ON SAVE')

    def save(self, *args, **kwargs):
        self.customer = self.company + '  [' + self.unique_id + ']'
        super(Company, self).save(*args, **kwargs)

    def __str__(self):
        return self.customer


class BuildOrder(models.Model):
    order_num = models.CharField(max_length=64)
    customer = models.CharField(max_length=64)
    product = models.CharField(max_length=128)
    quantity = models.IntegerField()
    from_stock = models.IntegerField(default=0)
    to_make = models.IntegerField(default=quantity)
    original_pk = models.CharField(max_length=8, default=0)
    max_lead_time = models.IntegerField(default=1)
    part_total = models.IntegerField(default=1)

    class Meta:
        ordering = ['max_lead_time']

    def __str__(self):
        return self.product


class Alert(models.Model):
    audio_alert_active = models.BooleanField(default=True)
    touched = models.BooleanField(default=True)
    last_touched = models.DateTimeField(auto_now=True)

    def __bool__(self):
        return self.audio_alert_active
